@@ -35,6 +35,7 @@
 #include <bluetooth-client-private.h>
 #include <bluetooth-killswitch.h>
 #include <bluetooth-chooser.h>
+#include <bluetooth-chooser-private.h>
 #include <bluetooth-plugin-manager.h>
 
 #include "adapter.h"
@@ -43,9 +44,11 @@
 static BluetoothClient *client;
 static GtkTreeModel *adapter_model;
 static BluetoothKillswitch *killswitch;
+static GtkNotebook *notebook;
 
-#define KILLSWITCH_PAGE_NUM(n) (gtk_notebook_get_n_pages (n) - 2)
-#define NO_ADAPTERS_PAGE_NUM(n) (-1)
+#define KILLSWITCH_PAGE_NUM(n) (gtk_notebook_get_n_pages (n) - 3)
+#define NO_ADAPTERS_PAGE_NUM(n) (gtk_notebook_get_n_pages (n) - 2)
+#define KILLSWITCHED_PAGE_NUM(n) (-1)
 
 typedef struct adapter_data adapter_data;
 struct adapter_data {
@@ -83,29 +86,10 @@ static void unblock_signals(adapter_data *adapter)
 
 static void discoverable_changed_cb(GtkWidget *button, gpointer user_data)
 {
-	adapter_data *adapter = user_data;
-	GValue discoverable = { 0 };
-	GValue timeout = { 0 };
-
-	g_value_init(&discoverable, G_TYPE_BOOLEAN);
-	g_value_init(&timeout, G_TYPE_UINT);
-
+	/* Note that this would be broken if we showed more than
+	 * one adapter, but we don't care */
 	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (button), FALSE);
-	g_value_set_boolean(&discoverable, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
-	g_value_set_uint(&timeout, 0);
-
-	dbus_g_proxy_call(adapter->proxy, "SetProperty", NULL,
-					G_TYPE_STRING, "Discoverable",
-					G_TYPE_VALUE, &discoverable,
-					G_TYPE_INVALID, G_TYPE_INVALID);
-
-	dbus_g_proxy_call(adapter->proxy, "SetProperty", NULL,
-					G_TYPE_STRING, "DiscoverableTimeout",
-					G_TYPE_VALUE, &timeout,
-					G_TYPE_INVALID, G_TYPE_INVALID);
-
-	g_value_unset(&discoverable);
-	g_value_unset(&timeout);
+	bluetooth_client_set_discoverable (client, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
 }
 
 static void name_callback(GtkWidget *editable, gpointer user_data)
@@ -169,75 +153,22 @@ static void wizard_callback(GtkWidget *button, gpointer user_data)
 		g_printerr("Couldn't execute command: %s\n", command);
 }
 
-static gboolean show_confirm_dialog(const char *name)
-{
-	GtkWidget *dialog;
-	gint response;
-
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-					 _("Remove '%s' from the list of devices?"), name);
-	g_object_set (G_OBJECT (dialog), "secondary-text",
-		      _("If you remove the device, you will have to set it up again before next use."),
-		      NULL);
-
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_REMOVE, GTK_RESPONSE_ACCEPT);
-
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (dialog);
-
-	if (response == GTK_RESPONSE_ACCEPT)
-		return TRUE;
-
-	return FALSE;
-}
-
 static void remove_callback(GtkWidget *button, gpointer user_data)
 {
 	adapter_data *adapter = user_data;
-	GValue value = { 0, };
-	DBusGProxy *device;
-	const char *path;
-	char *address, *name;
+	char *address;
 
-	if (bluetooth_chooser_get_selected_device_info (BLUETOOTH_CHOOSER (adapter->chooser),
-							"proxy", &value) == FALSE) {
-		return;
-	}
 	address = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (adapter->chooser));
-	name = bluetooth_chooser_get_selected_device_name (BLUETOOTH_CHOOSER (adapter->chooser));
-	if (address == NULL || name == NULL) {
-		g_value_unset (&value);
+	if (address == NULL) {
 		g_free (address);
-		g_free (name);
 		return;
 	}
-	device = g_value_dup_object (&value);
-	g_value_unset (&value);
 
-	if (device == NULL)
-		return;
-
-	path = dbus_g_proxy_get_path(device);
-
-	if (show_confirm_dialog (name) == TRUE) {
-		GError *err = NULL;
-		if (dbus_g_proxy_call (adapter->proxy, "RemoveDevice", &err,
-				       DBUS_TYPE_G_OBJECT_PATH, path,
-				       G_TYPE_INVALID, G_TYPE_INVALID) == FALSE) {
-			g_warning ("Failed to remove device %s: %s", address,
-				   err->message);
-			g_error_free (err);
-		} else {
-			bluetooth_plugin_manager_device_deleted (address);
-		}
-	}
+	if (bluetooth_chooser_remove_selected_device (BLUETOOTH_CHOOSER (adapter->chooser)))
+		bluetooth_plugin_manager_device_deleted (address);
+		
 
 	g_free (address);
-	g_free (name);
-	g_object_unref(device);
 }
 
 static void disconnect_callback(GtkWidget *button, gpointer user_data)
@@ -268,9 +199,16 @@ static void
 set_current_page (GtkNotebook *notebook)
 {
 	if (gtk_tree_model_iter_n_children (adapter_model, NULL) == 0) {
-		if (bluetooth_killswitch_has_killswitches (killswitch) != FALSE) {
+		KillswitchState state;
+
+		state = bluetooth_killswitch_get_state (killswitch);
+		if (bluetooth_killswitch_has_killswitches (killswitch) != FALSE &&
+		    state != KILLSWITCH_STATE_HARD_BLOCKED) {
 			gtk_notebook_set_current_page (notebook,
 						       KILLSWITCH_PAGE_NUM(notebook));
+		} else if (state == KILLSWITCH_STATE_HARD_BLOCKED) {
+			gtk_notebook_set_current_page (notebook,
+						       KILLSWITCHED_PAGE_NUM(notebook));
 		} else {
 			gtk_notebook_set_current_page (notebook,
 						       NO_ADAPTERS_PAGE_NUM(notebook));
@@ -345,7 +283,7 @@ static void create_adapter(adapter_data *adapter)
 	gtk_box_pack_start(GTK_BOX(mainbox), vbox, FALSE, TRUE, 0);
 
 	/* The discoverable checkbox */
-	button = gtk_check_button_new_with_mnemonic (_("Make computer _discoverable"));
+	button = gtk_check_button_new_with_mnemonic (_("Make computer _visible"));
 	if (powered == FALSE)
 		discoverable = FALSE;
 	if (discoverable != FALSE && timeout == 0)
@@ -480,6 +418,7 @@ static void create_adapter(adapter_data *adapter)
 static void update_visibility(adapter_data *adapter)
 {
 	gboolean inconsistent, enabled;
+
 	block_signals(adapter);
 
 	/* Switch off a few widgets */
@@ -703,6 +642,7 @@ killswitch_state_changed (BluetoothKillswitch *killswitch,
 {
 	if (state != KILLSWITCH_STATE_UNBLOCKED)
 		g_timeout_add_seconds (3, set_sensitive_now, user_data);
+	set_current_page (GTK_NOTEBOOK (notebook));
 }
 
 static void
@@ -745,7 +685,7 @@ create_killswitch_page (GtkNotebook *notebook)
 }
 
 static void
-create_no_adapter_page (GtkNotebook *notebook)
+create_no_adapter_page (GtkNotebook *notebook, const char *filename)
 {
 	GtkWidget *mainbox;
 	GtkWidget *vbox;
@@ -753,11 +693,15 @@ create_no_adapter_page (GtkNotebook *notebook)
 	GtkBuilder *xml;
 
 	xml = gtk_builder_new ();
-	if (gtk_builder_add_from_file (xml, "properties-no-adapter.ui", NULL) == 0) {
-		if (gtk_builder_add_from_file (xml, PKGDATADIR "/properties-no-adapter.ui", NULL) == 0) {
+	if (gtk_builder_add_from_file (xml, filename, NULL) == 0) {
+		char *path;
+		path = g_build_filename (PKGDATADIR, filename, NULL);
+		if (gtk_builder_add_from_file (xml, path, NULL) == 0) {
+			g_free (path);
 			g_warning ("Failed to load properties-no-adapter.ui");
 			return;
 		}
+		g_free (path);
 	}
 	vbox = GTK_WIDGET (gtk_builder_get_object (xml, "table1"));
 
@@ -774,13 +718,15 @@ create_no_adapter_page (GtkNotebook *notebook)
 	gtk_notebook_append_page(notebook, mainbox, NULL);
 }
 
-void setup_adapter(GtkNotebook *notebook)
+void setup_adapter(GtkNotebook *_notebook)
 {
 	killswitch = bluetooth_killswitch_new ();
+	notebook = _notebook;
 
 	/* Create our static pages first */
 	create_killswitch_page (notebook);
-	create_no_adapter_page (notebook);
+	create_no_adapter_page (notebook, "properties-no-adapter.ui");
+	create_no_adapter_page (notebook, "properties-killed-adapter.ui");
 
 	client = bluetooth_client_new();
 
